package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.base.utils.BeanUtils;
import com.zanxiang.game.module.manage.pojo.dto.*;
import com.zanxiang.game.module.manage.pojo.params.UserListParam;
import com.zanxiang.game.module.manage.pojo.params.UserUpdateParam;
import com.zanxiang.game.module.manage.pojo.vo.GameUserVO;
import com.zanxiang.game.module.manage.pojo.vo.UserCardVO;
import com.zanxiang.game.module.manage.pojo.vo.UserListVO;
import com.zanxiang.game.module.manage.pojo.vo.UserVO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.Agent;
import com.zanxiang.game.module.mybatis.entity.User;
import com.zanxiang.game.module.mybatis.mapper.UserMapper;
import com.zanxiang.module.util.encryption.Md5Util;
import com.zanxiang.module.util.exception.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户玩家
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private IGameService gameService;

    @Autowired
    private IGameUserService gameUserService;

    @Autowired
    private IUserService userService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IGameCategoryService gameCategoryService;

    @Autowired
    private IUserCardService userCardService;

    @Autowired
    private IAgentService agentService;

    @Override
    public IPage<UserListVO> list(UserListParam param) {
        //渠道列表
        List<AgentDTO> agentDTOList = agentService.listUserAgent(param.getAccountId(), param.getPitcherId(), param.getChannelId());
        //根据条件, 匹配渠道
        if (CollectionUtils.isEmpty(agentDTOList) && !SecurityUtil.isAdmin()) {
            return new Page<>();
        }
        //渠道id列表
        List<Long> agentIdList = agentDTOList.stream().map(AgentDTO::getId).collect(Collectors.toList());
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        //玩家条件处理
        Map<Long, UserDTO> userMap = userService.userCondition(param.getUserId(), agentIdList, param.getUserName(),
                param.getNickname(), param.getRegIp());
        //根据条件, 匹配不到玩家
        if (userMap != null && userMap.isEmpty()) {
            return new Page<>();
        }
        //相关用户id
        Set<Long> userIdSet = userMap == null ? null : userMap.keySet();
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        Map<Long, AgentDTO> agentMap = agentDTOList.stream().collect(Collectors.toMap(AgentDTO::getId, Function.identity()));
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        //执行查询
        return page(param.toPage(), new QueryWrapper<User>().lambda()
                .eq(param.getUserId() != null, User::getId, param.getUserId())
                .like(Strings.isNotBlank(param.getUserName()), User::getUsername, param.getUserName())
                .like(Strings.isNotBlank(param.getNickname()), User::getNickname, param.getNickname())
                .eq(Strings.isNotBlank(param.getMobile()), User::getMobile, param.getMobile())
                .in(CollectionUtils.isNotEmpty(userIdSet), User::getId, userIdSet)
                .isNotNull(Objects.equals(param.getIsBindMobile(), Boolean.TRUE), User::getMobile)
                .isNull(Objects.equals(param.getIsBindMobile(), Boolean.FALSE), User::getMobile)
                .eq(Objects.equals(param.getIsAuth(), Boolean.FALSE), User::getAuthentication, 0)
                .ne(Objects.equals(param.getIsAuth(), Boolean.TRUE), User::getAuthentication, 0)
                .in(CollectionUtils.isNotEmpty(gameMap), User::getGameId, CollectionUtils.isEmpty(gameMap) ? null : gameMap.keySet())
                .ge(param.getBeginDate() != null, User::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, User::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, User::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, User::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), User::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), User::getRechargeCount, 0)
                .eq(param.getStatus() != null, User::getStatus, param.getStatus())
                .eq(param.getChannelId() != null, User::getLastAgentId, param.getChannelId())
                .orderByDesc(User::getCreateTime)
        ).convert(u -> this.toVo(u, userMap, gameMap, cpMap, gameCategoryMap, agentMap));
    }

    private UserListVO toVo(User user, Map<Long, UserDTO> userMap, Map<Long, GameDTO> gameMap, Map<Long, CpDTO> cpMap,
                            Map<Long, GameCategoryDTO> gameCategoryMap, Map<Long, AgentDTO> agentMap) {
        UserListVO userListVO = BeanUtils.copy(user, UserListVO.class);
        if (Objects.isNull(userListVO)) {
            return null;
        }
        userListVO.setMobile(user.getShowPhoneNum());
        UserDTO userDTO = CollectionUtils.isEmpty(userMap) ? null : userMap.get(userListVO.getId());
        if (userDTO == null) {
            userDTO = this.getById(userListVO.getId());
        }
        if (userDTO != null) {
            userListVO.setRegIp(userDTO.getIp());
            userListVO.setUsername(userDTO.getUsername());
            userListVO.setNickname(userDTO.getNickname());
            userListVO.setRegAgentId(userDTO.getRegAgentId());
            //注册渠道
            AgentDTO regAgentDTO = agentMap.get(userDTO.getRegAgentId());

            if (regAgentDTO != null) {
                userListVO.setRegAgentName(regAgentDTO.getAgentName());
            }
            userListVO.setAgentId(userDTO.getLastAgentId());
            //最新渠道
            AgentDTO lastAgentDTO = agentMap.get(userDTO.getLastAgentId());
            if (lastAgentDTO != null) {
                userListVO.setAgentName(lastAgentDTO.getAgentName());
                userListVO.setAccountId(lastAgentDTO.getId());
                userListVO.setPitcherId(lastAgentDTO.getCreateBy());
                userListVO.setPitcherName(lastAgentDTO.getCreateByName());
                userListVO.setAccountType(lastAgentDTO.getAccountType());
            }
        }
        GameDTO gameDTO = CollectionUtils.isEmpty(gameMap) ? null : gameMap.get(userListVO.getGameId());
        if (gameDTO == null) {
            gameDTO = gameService.getById(userListVO.getGameId());
        }
        if (gameDTO != null) {
            userListVO.setGameName(gameDTO.getName());
            userListVO.setGameCategoryId(gameDTO.getCategory());
            userListVO.setGameCategoryName(gameCategoryMap.get(gameDTO.getCategory()) == null ? null : gameCategoryMap.get(gameDTO.getCategory()).getName());
            userListVO.setCpId(gameDTO.getCpId());
            userListVO.setCpName(cpMap.get(gameDTO.getCpId()) == null ? null : cpMap.get(gameDTO.getCpId()).getCpName());
        }
        return userListVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean userUpdate(UserUpdateParam param) {
        //用户信息更新
        if (Strings.isNotBlank(param.getMobile()) || Strings.isNotBlank(param.getAliPay()) || Strings.isNotBlank(param.getPassword())) {
            super.update(new LambdaUpdateWrapper<User>()
                    .eq(User::getId, param.getUserId())
                    .eq(User::getUsername, param.getUsername())
                    .set(Strings.isNotBlank(param.getMobile()) && !param.getMobile().contains("*"), User::getMobile, param.getMobile())
                    .set(Strings.isNotBlank(param.getAliPay()), User::getAliPay, param.getAliPay())
                    .set(Strings.isNotBlank(param.getPassword()), User::getPassword, this.cmfPassword(param.getPassword()))
                    .set(User::getUpdateTime, LocalDateTime.now()));
        }
        return Boolean.TRUE;
    }

    private String cmfPassword(String password) {
        String passwordSalt = password + "ZX_PASSWORD_SALT";
        return "###" + Md5Util.encrypt32(Md5Util.encrypt32(passwordSalt));
    }

    @Override
    public UserVO getUserInfo(Long userId) {
        //查询用户信息
        User user = super.getById(userId);
        UserVO userVO = BeanUtils.copy(user, UserVO.class);
        if (userVO == null) {
            throw new BaseException("用户信息不存在!");
        }
        //查询用户拓展信息
        UserCardVO userCardVO = userCardService.getByUserId(userId);
        if (userCardVO == null) {
            throw new BaseException("用户实名信息不存在!");
        }
        //拓展信息补充
        userVO.setRealName(userCardVO.getCardName());
        userVO.setBirthday(userCardVO.getBirthday());
        userVO.setSex(userCardVO.getSex());
        userVO.setIdCard(userCardVO.getCardId());
        userVO.setRegMobile(user.getShowPhoneNum());
        userVO.setMobile(user.getShowPhoneNum());
        userVO.setRegEmail(user.getEmail());
        userVO.setRegIp(user.getIp());
        userVO.setDeviceId(user.getMac());
        //注册游戏
        GameDTO gameDTO = gameService.getById(user.getGameId());
        if (gameDTO != null) {
            userVO.setRegGame(gameDTO.getName());
        }
        //渠道
        Agent agent = agentService.getById(user.getLastAgentId());
        if (agent != null) {
            userVO.setChannelName(agent.getAgentName());
            userVO.setRegFrom(agent.getAccountType());
        }
        //vip等级
        GameUserVO maxVipGameUser = gameUserService.getMaxVipGameUser(userId);
        if (maxVipGameUser != null) {
            userVO.setVipMax(maxVipGameUser.getRoleVipMax());
        }
        return userVO;
    }

    @Override
    public Map<Long, UserDTO> userCondition(Long userId, List<Long> agentIdList, String userName, String nickname, String regIp) {
        //用户信息条件处理
        if (userId == null && CollectionUtils.isEmpty(agentIdList) && Strings.isBlank(userName)
                && Strings.isBlank(nickname) && Strings.isBlank(regIp)) {
            return null;
        }
        //查询
        List<User> userList = super.list(new LambdaQueryWrapper<User>()
                .eq(userId != null, User::getId, userId)
                .in(CollectionUtils.isNotEmpty(agentIdList), User::getLastAgentId, agentIdList)
                .eq(Strings.isNotBlank(regIp), User::getIp, regIp)
                .like(Strings.isNotBlank(userName), User::getUsername, userName)
                .like(Strings.isNotBlank(nickname), User::getNickname, nickname)
                .select(User::getId, User::getUsername, User::getNickname, User::getIp, User::getCreateTime, User::getLastAgentId));
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        List<UserDTO> list = BeanUtils.copyList(userList, UserDTO.class);
        //构造返回
        return list.stream().collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    @Override
    public Map<Long, UserDTO> getByUserIds(List<Long> userIdList) {
        if (CollectionUtils.isEmpty(userIdList)) {
            return Collections.emptyMap();
        }
        //查询
        List<User> userList = super.listByIds(userIdList);
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        List<UserDTO> list = BeanUtils.copyList(userList, UserDTO.class);
        //构造返回
        return list.stream().collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    @Override
    public UserDTO getById(Long userId) {
        return BeanUtils.copy(super.getById(userId), UserDTO.class);
    }

}
